@@ -47,6 +47,7 @@
 #define INLINE_BLOB_SECRET              NMV_OVPN_TAG_SECRET
 #define INLINE_BLOB_TLS_AUTH            NMV_OVPN_TAG_TLS_AUTH
 #define INLINE_BLOB_TLS_CRYPT           NMV_OVPN_TAG_TLS_CRYPT
+#define INLINE_BLOB_TLS_CRYPT_V2        NMV_OVPN_TAG_TLS_CRYPT_V2
 
 const char *_nmovpn_test_temp_path = NULL;
 
@@ -791,7 +792,8 @@ do_import (const char *path, const char *contents, gsize contents_len, GError **
 	char *tmp, *tmp2;
 	const char *ta_direction = NULL, *secret_direction = NULL;
 	gboolean allow_ta_direction = FALSE, allow_secret_direction = FALSE;
-	gboolean have_certs, have_ca;
+	gboolean have_cert = FALSE, have_key = FALSE, have_ca = FALSE, have_pkcs12 = FALSE;
+	const char *cert_path = NULL, *key_path = NULL, *ca_path = NULL;
 	GSList *inline_blobs = NULL;
 	GSList *sl_iter;
 
@@ -897,18 +899,17 @@ do_import (const char *path, const char *contents, gsize contents_len, GError **
 		if (NM_IN_STRSET (params[0], NMV_OVPN_TAG_PROTO)) {
 			if (!args_params_check_nargs_n (params, 1, &line_error))
 				goto handle_line_error;
-			/* Valid parameters are "udp", "tcp-client" and "tcp-server".
+			/* Valid parameters are defined in shared/utils.h
 			 * 'tcp' isn't technically valid, but it used to be accepted so
 			 * we'll handle it here anyway.
 			 */
-			if (nm_streq (params[1], "udp")) {
-				/* ignore; udp is default */
-			} else if (NM_IN_STRSET (params[1], "tcp-client", "tcp-server", "tcp"))
-				setting_vpn_add_data_item (s_vpn, NM_OPENVPN_KEY_PROTO_TCP, "yes");
-			else {
-				line_error = args_params_error_message_invalid_arg (params, 1);
+			if (!NM_IN_STRSET (params[1], NMOVPN_PROTCOL_TYPES)) {
+				line_error = g_strdup_printf (_("proto expects protocol type like “udp” or “tcp”"));
 				goto handle_line_error;
 			}
+			if (!NM_IN_STRSET (params[1], "udp", "udp4", "udp6")) {
+				setting_vpn_add_data_item (s_vpn, NM_OPENVPN_KEY_PROTO_TCP, "yes");
+			}
 			continue;
 		}
 
@@ -998,6 +999,19 @@ do_import (const char *path, const char *contents, gsize contents_len, GError **
 			continue;
 		}
 
+		if (NM_IN_STRSET (params[0], NMV_OVPN_TAG_ALLOW_COMPRESSION)) {
+			if (!args_params_check_nargs_minmax (params, 1, 1, &line_error))
+				goto handle_line_error;
+
+			if (!NM_IN_STRSET (params[1], "yes", "no", "asym")) {
+				line_error = g_strdup_printf (_("allow-compression: invalid argument"));
+				goto handle_line_error;
+			}
+
+			setting_vpn_add_data_item (s_vpn, NM_OPENVPN_KEY_ALLOW_COMPRESSION, params[1]);
+			continue;
+		}
+
 		if (NM_IN_STRSET (params[0], NMV_OVPN_TAG_COMP_LZO)) {
 			const char *v;
 
@@ -1186,6 +1200,20 @@ do_import (const char *path, const char *contents, gsize contents_len, GError **
 			continue;
 		}
 
+		if (NM_IN_STRSET (params[0], NMV_OVPN_TAG_REMOTE_RANDOM_HOSTNAME)) {
+			if (!args_params_check_nargs_n (params, 0, &line_error))
+				goto handle_line_error;
+			setting_vpn_add_data_item (s_vpn, NM_OPENVPN_KEY_REMOTE_RANDOM_HOSTNAME, "yes");
+			continue;
+		}
+
+		if (NM_IN_STRSET (params[0], NMV_OVPN_TAG_ALLOW_PULL_FQDN)) {
+			if (!args_params_check_nargs_n (params, 0, &line_error))
+				goto handle_line_error;
+			setting_vpn_add_data_item (s_vpn, NM_OPENVPN_KEY_ALLOW_PULL_FQDN, "yes");
+			continue;
+		}
+
 		if (NM_IN_STRSET (params[0], NMV_OVPN_TAG_TUN_IPV6)) {
 			if (!args_params_check_nargs_n (params, 0, &line_error))
 				goto handle_line_error;
@@ -1193,6 +1221,13 @@ do_import (const char *path, const char *contents, gsize contents_len, GError **
 			continue;
 		}
 
+		if (NM_IN_STRSET (params[0], NMV_OVPN_TAG_PUSH_PEER_INFO)) {
+			if (!args_params_check_nargs_n (params, 0, &line_error))
+				goto handle_line_error;
+			setting_vpn_add_data_item (s_vpn, NM_OPENVPN_KEY_PUSH_PEER_INFO, "yes");
+			continue;
+		}
+
 		if (NM_IN_STRSET (params[0], NMV_OVPN_TAG_PORT, NMV_OVPN_TAG_RPORT)) {
 			if (!args_params_check_nargs_n (params, 1, &line_error))
 				goto handle_line_error;
@@ -1221,16 +1256,24 @@ do_import (const char *path, const char *contents, gsize contents_len, GError **
 			continue;
 		}
 
-		if (NM_IN_STRSET (params[0], NMV_OVPN_TAG_TLS_VERSION_MIN)){
-			if (!args_params_check_nargs_n (params, 1, &line_error))
+		if (NM_IN_STRSET (params[0], NMV_OVPN_TAG_TLS_VERSION_MIN)) {
+			if (!args_params_check_nargs_minmax (params, 1, 2, &line_error))
 				goto handle_line_error;
 			if (!args_params_check_arg_utf8 (params, 1, NULL, &line_error))
 				goto handle_line_error;
 			setting_vpn_add_data_item (s_vpn, NM_OPENVPN_KEY_TLS_VERSION_MIN, params[1]);
+			if (params[2]) {
+				if (nm_streq (params[2], "or-highest")) {
+					setting_vpn_add_data_item (s_vpn, NM_OPENVPN_KEY_TLS_VERSION_MIN_OR_HIGHEST, "yes");
+				} else {
+					line_error = g_strdup_printf (_("invalid keyword “%s” in tls-version-min"), params[2]);
+					goto handle_line_error;
+				}
+			}
 			continue;
 		}
 
-		if (NM_IN_STRSET (params[0], NMV_OVPN_TAG_TLS_VERSION_MAX)){
+		if (NM_IN_STRSET (params[0], NMV_OVPN_TAG_TLS_VERSION_MAX)) {
 			if (!args_params_check_nargs_n (params, 1, &line_error))
 				goto handle_line_error;
 			if (!args_params_check_arg_utf8 (params, 1, NULL, &line_error))
@@ -1247,7 +1290,8 @@ do_import (const char *path, const char *contents, gsize contents_len, GError **
 		                  NMV_OVPN_TAG_PKCS12,
 		                  NMV_OVPN_TAG_SECRET,
 		                  NMV_OVPN_TAG_TLS_AUTH,
-		                  NMV_OVPN_TAG_TLS_CRYPT)) {
+		                  NMV_OVPN_TAG_TLS_CRYPT,
+		                  NMV_OVPN_TAG_TLS_CRYPT_V2)) {
 			const char *file;
 			gs_free char *file_free = NULL;
 			gboolean can_have_direction;
@@ -1273,18 +1317,22 @@ do_import (const char *path, const char *contents, gsize contents_len, GError **
 				file = file_free = g_build_filename (default_path, file, NULL);
 
 			if (NM_IN_STRSET (params[0], NMV_OVPN_TAG_PKCS12)) {
-				setting_vpn_add_data_item_path (s_vpn, NM_OPENVPN_KEY_CA, file);
+				 /* OpenVPN allows --pkcs12 with external (PEM) --ca. Don't overwrite it with the PKCS#12 file. */
+				if (!have_ca)
+					setting_vpn_add_data_item_path (s_vpn, NM_OPENVPN_KEY_CA, file);
 				setting_vpn_add_data_item_path (s_vpn, NM_OPENVPN_KEY_CERT, file);
 				setting_vpn_add_data_item_path (s_vpn, NM_OPENVPN_KEY_KEY, file);
-			} else if (NM_IN_STRSET (params[0], NMV_OVPN_TAG_CA))
+				have_pkcs12 = TRUE;
+			} else if (NM_IN_STRSET (params[0], NMV_OVPN_TAG_CA)) {
 				setting_vpn_add_data_item_path (s_vpn, NM_OPENVPN_KEY_CA, file);
-			else if (NM_IN_STRSET (params[0], NMV_OVPN_TAG_EXTRA_CERTS))
-				setting_vpn_add_data_item_path (s_vpn, NM_OPENVPN_KEY_EXTRA_CERTS, file);
-			else if (NM_IN_STRSET (params[0], NMV_OVPN_TAG_CERT))
+				have_ca = TRUE;
+			} else if (NM_IN_STRSET (params[0], NMV_OVPN_TAG_CERT)) {
 				setting_vpn_add_data_item_path (s_vpn, NM_OPENVPN_KEY_CERT, file);
-			else if (NM_IN_STRSET (params[0], NMV_OVPN_TAG_KEY))
+				have_cert = TRUE;
+			} else if (NM_IN_STRSET (params[0], NMV_OVPN_TAG_KEY)) {
 				setting_vpn_add_data_item_path (s_vpn, NM_OPENVPN_KEY_KEY, file);
-			else if (NM_IN_STRSET (params[0], NMV_OVPN_TAG_SECRET)) {
+				have_key = TRUE;
+			} else if (NM_IN_STRSET (params[0], NMV_OVPN_TAG_SECRET)) {
 				setting_vpn_add_data_item_path (s_vpn, NM_OPENVPN_KEY_STATIC_KEY, file);
 				if (s_direction)
 					secret_direction = s_direction;
@@ -1297,6 +1345,10 @@ do_import (const char *path, const char *contents, gsize contents_len, GError **
 				allow_ta_direction = TRUE;
 			} else if (NM_IN_STRSET (params[0], NMV_OVPN_TAG_TLS_CRYPT))
 				setting_vpn_add_data_item_path (s_vpn, NM_OPENVPN_KEY_TLS_CRYPT, file);
+			else if (NM_IN_STRSET (params[0], NMV_OVPN_TAG_TLS_CRYPT_V2))
+				setting_vpn_add_data_item_path (s_vpn, NM_OPENVPN_KEY_TLS_CRYPT_V2, file);
+			else if (NM_IN_STRSET (params[0], NMV_OVPN_TAG_EXTRA_CERTS))
+				setting_vpn_add_data_item_path (s_vpn, NM_OPENVPN_KEY_EXTRA_CERTS, file);
 			else
 				g_assert_not_reached ();
 			continue;
@@ -1422,6 +1474,7 @@ do_import (const char *path, const char *contents, gsize contents_len, GError **
 			in_addr_t gateway = 0;
 			guint32 prefix = 32;
 			gint64 metric = -1;
+			NMIPRoute *route;
 
 			if (!args_params_check_nargs_minmax (params, 1, 4, &line_error))
 				goto handle_line_error;
@@ -1457,26 +1510,9 @@ do_import (const char *path, const char *contents, gsize contents_len, GError **
 				continue;
 			}
 
-			{
-#if ((NETWORKMANAGER_COMPILATION) & NM_NETWORKMANAGER_COMPILATION_WITH_LIBNM_UTIL)
-				NMIP4Route *route;
-
-				route = nm_ip4_route_new ();
-				nm_ip4_route_set_dest (route, network);
-				nm_ip4_route_set_prefix (route, prefix);
-				nm_ip4_route_set_next_hop (route, gateway);
-				if (metric >= 0)
-					nm_ip4_route_set_metric (route, metric);
-				nm_setting_ip4_config_add_route (s_ip4, route);
-				nm_ip4_route_unref (route);
-#else
-				NMIPRoute *route;
-
-				route = nm_ip_route_new_binary (AF_INET, &network, prefix, params[3] ? &gateway : NULL, metric, NULL);
-				nm_setting_ip_config_add_route (s_ip4, route);
-				nm_ip_route_unref (route);
-#endif
-			}
+			route = nm_ip_route_new_binary (AF_INET, &network, prefix, params[3] ? &gateway : NULL, metric, NULL);
+			nm_setting_ip_config_add_route (s_ip4, route);
+			nm_ip_route_unref (route);
 		}
 
 		if (params[0][0] == '<' && params[0][strlen (params[0]) - 1] == '>') {
@@ -1490,21 +1526,27 @@ do_import (const char *path, const char *contents, gsize contents_len, GError **
 			GString *blob_data;
 			InlineBlobData *inline_blob_data;
 
-			if (nm_streq (token, INLINE_BLOB_CA))
+			if (nm_streq (token, INLINE_BLOB_CA)) {
 				key = NM_OPENVPN_KEY_CA;
-			else if (nm_streq (token, INLINE_BLOB_CERT))
+				have_ca = TRUE;
+			} else if (nm_streq (token, INLINE_BLOB_CERT)) {
 				key = NM_OPENVPN_KEY_CERT;
-			else if (nm_streq (token, INLINE_BLOB_KEY))
+				have_cert = TRUE;
+			} else if (nm_streq (token, INLINE_BLOB_KEY)) {
 				key = NM_OPENVPN_KEY_KEY;
-			else if (nm_streq (token, INLINE_BLOB_EXTRA_CERTS))
+				have_key = TRUE;
+			} else if (nm_streq (token, INLINE_BLOB_PKCS12)) {
+				is_base64 = TRUE;
+				key = NULL;
+				have_pkcs12 = TRUE;
+			} else if (nm_streq (token, INLINE_BLOB_EXTRA_CERTS))
 				key = NM_OPENVPN_KEY_EXTRA_CERTS;
 			else if (nm_streq (token, INLINE_BLOB_CRL_VERIFY))
 				key = NM_OPENVPN_KEY_CRL_VERIFY_FILE;
-			else if (nm_streq (token, INLINE_BLOB_PKCS12)) {
-				is_base64 = TRUE;
-				key = NULL;
-			} else if (nm_streq (token, INLINE_BLOB_TLS_CRYPT))
+			else if (nm_streq (token, INLINE_BLOB_TLS_CRYPT))
 				key = NM_OPENVPN_KEY_TLS_CRYPT;
+			else if (nm_streq (token, INLINE_BLOB_TLS_CRYPT_V2))
+				key = NM_OPENVPN_KEY_TLS_CRYPT_V2;
 			else if (nm_streq (token, INLINE_BLOB_TLS_AUTH)) {
 				key = NM_OPENVPN_KEY_TA;
 				allow_ta_direction = TRUE;
@@ -1621,28 +1663,77 @@ handle_line_error:
 		g_set_error_literal (error,
 		                     NMV_EDITOR_PLUGIN_ERROR,
 		                     NMV_EDITOR_PLUGIN_ERROR_FILE_NOT_VPN,
-		                     _("The file to import wasn’t a valid OpenVPN configure (no remote)"));
+		                     _("The file to import wasn’t a valid OpenVPN configuration (no remote)"));
 		goto out_error;
 	}
 
-	have_certs = FALSE;
-	have_ca = FALSE;
+	/* Validate PKCS#12/PEM/CA settings. PKCS#12 and PEM cannot be mixed,
+	 * with the exception of PKCS#12 cert + PEM CA. */
+	if (have_client && have_pkcs12) {
+		if (have_cert || have_key) {
+			g_set_error_literal (error,
+			                     NMV_EDITOR_PLUGIN_ERROR,
+			                     NMV_EDITOR_PLUGIN_ERROR_FILE_NOT_VPN,
+			                     _("The file to import wasn’t a valid OpenVPN configuration (--pkcs12 can not be used with --cert or --key)"));
+			goto out_error;
+		}
 
-	if (nm_setting_vpn_get_data_item (s_vpn, NM_OPENVPN_KEY_CA))
-		have_ca = TRUE;
+		if (have_ca) {
+			ca_path = nm_setting_vpn_get_data_item (s_vpn, NM_OPENVPN_KEY_CA);
+			if (is_pkcs12 (ca_path)) {
+				g_set_error_literal (error,
+				                     NMV_EDITOR_PLUGIN_ERROR,
+				                     NMV_EDITOR_PLUGIN_ERROR_FILE_NOT_VPN,
+				                     _("The file to import wasn’t a valid OpenVPN configuration (--ca can not be PKCS#12 format)"));
+				goto out_error;
+			}
+		}
+	} else if (have_client && !have_pkcs12) {
+		if (!have_ca) {
+			g_set_error_literal (error,
+			                     NMV_EDITOR_PLUGIN_ERROR,
+			                     NMV_EDITOR_PLUGIN_ERROR_FILE_NOT_VPN,
+			                     _("The file to import wasn’t a valid OpenVPN configuration (missing --ca)"));
+			goto out_error;
+		}
 
-	if (   have_ca
-	    && nm_setting_vpn_get_data_item (s_vpn, NM_OPENVPN_KEY_CERT)
-	    && nm_setting_vpn_get_data_item (s_vpn, NM_OPENVPN_KEY_KEY))
-		have_certs = TRUE;
+		ca_path = nm_setting_vpn_get_data_item (s_vpn, NM_OPENVPN_KEY_CA);
+
+		if ((have_cert || have_key) && !(have_cert && have_key)) {
+			g_set_error_literal (error,
+			                     NMV_EDITOR_PLUGIN_ERROR,
+			                     NMV_EDITOR_PLUGIN_ERROR_FILE_NOT_VPN,
+			                     _("The file to import wasn’t a valid OpenVPN configuration (if you use one of --cert or --key, you must use them both)"));
+			goto out_error;
+		}
+
+		cert_path = nm_setting_vpn_get_data_item (s_vpn, NM_OPENVPN_KEY_CERT);
+		key_path = nm_setting_vpn_get_data_item (s_vpn, NM_OPENVPN_KEY_KEY);
+
+		if (cert_path && is_pkcs12 (cert_path)) {
+			g_set_error_literal (error,
+			                     NMV_EDITOR_PLUGIN_ERROR,
+			                     NMV_EDITOR_PLUGIN_ERROR_FILE_NOT_VPN,
+			                     _("The file to import wasn’t a valid OpenVPN configuration (--cert can not be PKCS#12 format.)"));
+			goto out_error;
+		}
+
+		if (key_path && is_pkcs12 (key_path)) {
+			g_set_error_literal (error,
+			                     NMV_EDITOR_PLUGIN_ERROR,
+			                     NMV_EDITOR_PLUGIN_ERROR_FILE_NOT_VPN,
+			                     _("The file to import wasn’t a valid OpenVPN configuration (--key can not be PKCS#12 format.)"));
+			goto out_error;
+		}
+	}
 
 	/* Determine connection type */
 	if (have_pass) {
-		if (have_certs)
+		if (have_cert || have_pkcs12)
 			ctype = NM_OPENVPN_CONTYPE_PASSWORD_TLS;
 		else if (have_ca)
 			ctype = NM_OPENVPN_CONTYPE_PASSWORD;
-	} else if (have_certs) {
+	} else if (have_cert || have_pkcs12) {
 		ctype = NM_OPENVPN_CONTYPE_TLS;
 	} else if (have_sk)
 		ctype = NM_OPENVPN_CONTYPE_STATIC_KEY;
@@ -1659,15 +1750,12 @@ handle_line_error:
 		                             NM_SETTING_SECRET_FLAG_AGENT_OWNED,
 		                             NULL);
 	}
-	if (have_certs) {
+	if (have_key || have_pkcs12) {
 		gs_free char *key_path_free = NULL;
-		const char *key_path;
 
-		key_path = nm_setting_vpn_get_data_item (s_vpn, NM_OPENVPN_KEY_KEY);
 		if (is_encrypted (nm_utils_str_utf8safe_unescape (key_path, &key_path_free))) {
 			/* If there should be a private key password, default it to
-			 * being agent-owned.
-			 */
+			 * being agent-owned. */
 			nm_setting_set_secret_flags (NM_SETTING (s_vpn),
 			                             NM_OPENVPN_KEY_CERTPASS,
 			                             NM_SETTING_SECRET_FLAG_AGENT_OWNED,
@@ -1913,7 +2001,10 @@ do_export_create (NMConnection *connection, const char *path, GError **error)
 
 	f = g_string_sized_new (512);
 
-	args_write_line (f, NMV_OVPN_TAG_CLIENT);
+	if (NM_IN_STRSET (connection_type, NM_OPENVPN_CONTYPE_TLS,
+	                                   NM_OPENVPN_CONTYPE_PASSWORD,
+	                                   NM_OPENVPN_CONTYPE_PASSWORD_TLS))
+		args_write_line (f, NMV_OVPN_TAG_CLIENT);
 
 	/* 'remote' */
 	gw_list = g_strsplit_set (gateways, " \t,", 0);
@@ -1941,9 +2032,18 @@ do_export_create (NMConnection *connection, const char *path, GError **error)
 	if (nm_streq0 (nm_setting_vpn_get_data_item (s_vpn, NM_OPENVPN_KEY_REMOTE_RANDOM), "yes"))
 		args_write_line (f, NMV_OVPN_TAG_REMOTE_RANDOM);
 
+	if (nm_streq0 (nm_setting_vpn_get_data_item (s_vpn, NM_OPENVPN_KEY_REMOTE_RANDOM_HOSTNAME), "yes"))
+		args_write_line (f, NMV_OVPN_TAG_REMOTE_RANDOM_HOSTNAME);
+
+	if (nm_streq0 (nm_setting_vpn_get_data_item (s_vpn, NM_OPENVPN_KEY_ALLOW_PULL_FQDN), "yes"))
+		args_write_line (f, NMV_OVPN_TAG_ALLOW_PULL_FQDN);
+
 	if (nm_streq0 (nm_setting_vpn_get_data_item (s_vpn, NM_OPENVPN_KEY_TUN_IPV6), "yes"))
 		args_write_line (f, NMV_OVPN_TAG_TUN_IPV6);
 
+	if (nm_streq0 (nm_setting_vpn_get_data_item (s_vpn, NM_OPENVPN_KEY_PUSH_PEER_INFO), "yes"))
+		args_write_line (f, NMV_OVPN_TAG_PUSH_PEER_INFO);
+
 	{
 		gs_free char *cacert_free = NULL, *user_cert_free = NULL, *private_key_free = NULL;
 		const char *cacert = NULL, *user_cert = NULL, *private_key = NULL;
@@ -1967,18 +2067,21 @@ do_export_create (NMConnection *connection, const char *path, GError **error)
 				private_key = nm_utils_str_utf8safe_unescape (value, &private_key_free);
 		}
 
-		if (   cacert && user_cert && private_key
-		    && nm_streq (cacert, user_cert) && nm_streq (cacert, private_key)) {
-			/* Handle PKCS#12 (all certs are the same file) */
-			args_write_line (f, NMV_OVPN_TAG_PKCS12, cacert);
-		} else {
-			if (cacert)
-				args_write_line (f, NMV_OVPN_TAG_CA, cacert);
-			if (user_cert)
+		if (user_cert && private_key) {
+			if (   nm_streq (user_cert, private_key)
+			    && is_pkcs12 (user_cert))
+				args_write_line (f, NMV_OVPN_TAG_PKCS12, user_cert);
+			else {
 				args_write_line (f, NMV_OVPN_TAG_CERT, user_cert);
-			if (private_key)
 				args_write_line (f, NMV_OVPN_TAG_KEY, private_key);
+			}
 		}
+
+		/* The CA can be included in a PKCS#12 file above, or a separate file (PEM format).
+		 * If it's pkcs12 we can't use --ca as it only accepts PEM format. */
+		if (   cacert
+		    && !is_pkcs12 (cacert))
+			args_write_line (f, NMV_OVPN_TAG_CA, cacert);
 	}
 
 	if (NM_IN_STRSET (connection_type, NM_OPENVPN_CONTYPE_PASSWORD,
@@ -2007,18 +2110,23 @@ do_export_create (NMConnection *connection, const char *path, GError **error)
 
 	args_write_line_setting_value_int (f, NMV_OVPN_TAG_KEYSIZE, s_vpn, NM_OPENVPN_KEY_KEYSIZE);
 
-	value = nm_setting_vpn_get_data_item (s_vpn, NM_OPENVPN_KEY_COMP_LZO);
-	if (value) {
-		if (nm_streq (value, "no-by-default"))
-			value = "no";
-		args_write_line (f, NMV_OVPN_TAG_COMP_LZO, value);
-	}
+	args_write_line_setting_value (f, NMV_OVPN_TAG_ALLOW_COMPRESSION, s_vpn, NM_OPENVPN_KEY_ALLOW_COMPRESSION);
 
-	value = nm_setting_vpn_get_data_item (s_vpn, NM_OPENVPN_KEY_COMPRESS);
-	if (nm_streq0 (value, "yes"))
-		args_write_line (f, NMV_OVPN_TAG_COMPRESS);
-	else if (value)
-		args_write_line_setting_value (f, NMV_OVPN_TAG_COMPRESS, s_vpn, NM_OPENVPN_KEY_COMPRESS);
+	value = nm_setting_vpn_get_data_item (s_vpn, NM_OPENVPN_KEY_ALLOW_COMPRESSION);
+	if (!nm_streq0 (value, "no")) {
+		value = nm_setting_vpn_get_data_item (s_vpn, NM_OPENVPN_KEY_COMP_LZO);
+		if (value) {
+			if (nm_streq (value, "no-by-default"))
+				value = "no";
+			args_write_line (f, NMV_OVPN_TAG_COMP_LZO, value);
+		}
+
+		value = nm_setting_vpn_get_data_item (s_vpn, NM_OPENVPN_KEY_COMPRESS);
+		if (nm_streq0 (value, "yes"))
+			args_write_line (f, NMV_OVPN_TAG_COMPRESS);
+		else if (value)
+			args_write_line_setting_value (f, NMV_OVPN_TAG_COMPRESS, s_vpn, NM_OPENVPN_KEY_COMPRESS);
+	}
 
 	if (nm_streq0 (nm_setting_vpn_get_data_item (s_vpn, NM_OPENVPN_KEY_FLOAT), "yes"))
 		args_write_line (f, NMV_OVPN_TAG_FLOAT);
@@ -2122,9 +2230,21 @@ do_export_create (NMConnection *connection, const char *path, GError **error)
 			                 nm_utils_str_utf8safe_unescape (key, &s_free));
 		}
 
+		key = nm_setting_vpn_get_data_item (s_vpn, NM_OPENVPN_KEY_TLS_CRYPT_V2);
+		if (nmovpn_arg_is_set (key)) {
+			gs_free char *s_free = NULL;
+			args_write_line (f,
+			                 NMV_OVPN_TAG_TLS_CRYPT_V2,
+			                 nm_utils_str_utf8safe_unescape (key, &s_free));
+		}
+
 		key = nm_setting_vpn_get_data_item (s_vpn, NM_OPENVPN_KEY_TLS_VERSION_MIN);
-		if (nmovpn_arg_is_set (key))
-			args_write_line (f, NMV_OVPN_TAG_TLS_VERSION_MIN, key);
+		if (nmovpn_arg_is_set (key)) {
+			const char *or_highest = nm_setting_vpn_get_data_item (s_vpn, NM_OPENVPN_KEY_TLS_VERSION_MIN_OR_HIGHEST);
+
+			args_write_line (f, NMV_OVPN_TAG_TLS_VERSION_MIN, key,
+			                 nm_streq0 (or_highest, "yes") ? "or-highest" : NULL);
+		}
 
 		key = nm_setting_vpn_get_data_item (s_vpn, NM_OPENVPN_KEY_TLS_VERSION_MAX);
 		if (nmovpn_arg_is_set (key))
@@ -2194,11 +2314,7 @@ do_export_create (NMConnection *connection, const char *path, GError **error)
 
 	s_ip4 = nm_connection_get_setting_ip4_config (connection);
 	if (s_ip4) {
-#if ((NETWORKMANAGER_COMPILATION) & NM_NETWORKMANAGER_COMPILATION_WITH_LIBNM_UTIL)
-		num = nm_setting_ip4_config_get_num_routes (s_ip4);
-#else
 		num = nm_setting_ip_config_get_num_routes (s_ip4);
-#endif
 		for (i = 0; i < num; i++) {
 			char netmask_str[INET_ADDRSTRLEN] = { 0 };
 			const char *next_hop_str, *dest_str;
@@ -2206,31 +2322,13 @@ do_export_create (NMConnection *connection, const char *path, GError **error)
 			guint prefix;
 			guint64 metric;
 			char metric_buf[50];
+			NMIPRoute *route;
 
-#if ((NETWORKMANAGER_COMPILATION) & NM_NETWORKMANAGER_COMPILATION_WITH_LIBNM_UTIL)
-			char next_hop_str_buf[INET_ADDRSTRLEN] = { 0 };
-			char dest_str_buf[INET_ADDRSTRLEN] = { 0 };
-			in_addr_t dest, next_hop;
-			NMIP4Route *route = nm_setting_ip4_config_get_route (s_ip4, i);
-
-			dest = nm_ip4_route_get_dest (route);
-			inet_ntop (AF_INET, (const void *) &dest, dest_str_buf, sizeof (dest_str_buf));
-			dest_str = dest_str_buf;
-
-			next_hop = nm_ip4_route_get_next_hop (route);
-			inet_ntop (AF_INET, (const void *) &next_hop, next_hop_str_buf, sizeof (next_hop_str_buf));
-			next_hop_str = next_hop_str_buf;
-
-			prefix = nm_ip4_route_get_prefix (route);
-			metric = nm_ip4_route_get_metric (route);
-#else
-			NMIPRoute *route = nm_setting_ip_config_get_route (s_ip4, i);
-
+			route = nm_setting_ip_config_get_route (s_ip4, i);
 			dest_str = nm_ip_route_get_dest (route);
 			next_hop_str = nm_ip_route_get_next_hop (route) ? : "0.0.0.0",
 			prefix = nm_ip_route_get_prefix (route);
 			metric = nm_ip_route_get_metric (route);
-#endif
 			netmask = nm_utils_ip4_prefix_to_netmask (prefix);
 			inet_ntop (AF_INET, (const void *) &netmask, netmask_str, sizeof (netmask_str));
 
