@@ -29,6 +29,8 @@
 #define NM_DBUS_INTERFACE_OPENVPN  "org.freedesktop.NetworkManager.openvpn"
 #define NM_DBUS_PATH_OPENVPN       "/org/freedesktop/NetworkManager/openvpn"
 
+#define NM_OPENVPN_KEY_ALLOW_COMPRESSION         "allow-compression"
+#define NM_OPENVPN_KEY_ALLOW_PULL_FQDN           "allow-pull-fqdn"
 #define NM_OPENVPN_KEY_AUTH                      "auth"
 #define NM_OPENVPN_KEY_CA                        "ca"
 #define NM_OPENVPN_KEY_CERT                      "cert"
@@ -62,10 +64,12 @@
 #define NM_OPENVPN_KEY_PROXY_RETRY               "proxy-retry"
 #define NM_OPENVPN_KEY_PROXY_SERVER              "proxy-server"
 #define NM_OPENVPN_KEY_PROXY_TYPE                "proxy-type"
+#define NM_OPENVPN_KEY_PUSH_PEER_INFO            "push-peer-info"
 #define NM_OPENVPN_KEY_REMOTE                    "remote"
 #define NM_OPENVPN_KEY_REMOTE_CERT_TLS           "remote-cert-tls"
 #define NM_OPENVPN_KEY_REMOTE_IP                 "remote-ip"
 #define NM_OPENVPN_KEY_REMOTE_RANDOM             "remote-random"
+#define NM_OPENVPN_KEY_REMOTE_RANDOM_HOSTNAME    "remote-random-hostname"
 #define NM_OPENVPN_KEY_RENEG_SECONDS             "reneg-seconds"
 #define NM_OPENVPN_KEY_STATIC_KEY                "static-key"
 #define NM_OPENVPN_KEY_STATIC_KEY_DIRECTION      "static-key-direction"
@@ -74,8 +78,10 @@
 #define NM_OPENVPN_KEY_TA_DIR                    "ta-dir"
 #define NM_OPENVPN_KEY_TLS_CIPHER                "tls-cipher"
 #define NM_OPENVPN_KEY_TLS_CRYPT                 "tls-crypt"
+#define NM_OPENVPN_KEY_TLS_CRYPT_V2              "tls-crypt-v2"
 #define NM_OPENVPN_KEY_TLS_REMOTE                "tls-remote"
 #define NM_OPENVPN_KEY_TLS_VERSION_MIN           "tls-version-min"
+#define NM_OPENVPN_KEY_TLS_VERSION_MIN_OR_HIGHEST "tls-version-min-or-highest"
 #define NM_OPENVPN_KEY_TLS_VERSION_MAX           "tls-version-max"
 #define NM_OPENVPN_KEY_TUNNEL_MTU                "tunnel-mtu"
 #define NM_OPENVPN_KEY_TUN_IPV6                  "tun-ipv6"
