@@ -89,6 +89,7 @@ typedef enum {
 	OPENVPN_BINARY_VERSION_INVALID,
 	OPENVPN_BINARY_VERSION_UNKNOWN,
 	OPENVPN_BINARY_VERSION_2_3_OR_OLDER,
+	OPENVPN_BINARY_VERSION_2_4_OR_OLDER,
 	OPENVPN_BINARY_VERSION_2_4_OR_NEWER,
 } OpenvpnBinaryVersion;
 
@@ -138,6 +139,7 @@ typedef struct {
 } ValidProperty;
 
 static const ValidProperty valid_properties[] = {
+	{ NM_OPENVPN_KEY_ALLOW_COMPRESSION,         G_TYPE_STRING, 0, 0, FALSE },
 	{ NM_OPENVPN_KEY_ALLOW_PULL_FQDN,           G_TYPE_BOOLEAN, 0, 0, FALSE },
 	{ NM_OPENVPN_KEY_AUTH,                      G_TYPE_STRING, 0, 0, FALSE },
 	{ NM_OPENVPN_KEY_CA,                        G_TYPE_STRING, 0, 0, FALSE },
@@ -197,6 +199,7 @@ static const ValidProperty valid_properties[] = {
 	{ NM_OPENVPN_KEY_NOSECRET,                  G_TYPE_STRING, 0, 0, FALSE },
 	{ NM_OPENVPN_KEY_HTTP_PROXY_PASSWORD_FLAGS, G_TYPE_STRING, 0, 0, FALSE },
 	{ NM_OPENVPN_KEY_TLS_VERSION_MIN,           G_TYPE_STRING, 0, 0, FALSE },
+	{ NM_OPENVPN_KEY_TLS_VERSION_MIN_OR_HIGHEST,G_TYPE_BOOLEAN, 0, 0, FALSE },
 	{ NM_OPENVPN_KEY_TLS_VERSION_MAX,           G_TYPE_STRING, 0, 0, FALSE },
 	{ NULL,                                     G_TYPE_NONE, FALSE }
 };
@@ -364,8 +367,10 @@ args_add_vpn_certs (GPtrArray *args, NMSettingVpn *s_vpn)
 	if (nmovpn_arg_is_set (cert) && is_pkcs12 (cert))
 		args_add_strv (args, "--pkcs12", cert);
 	else {
-		args_add_strv (args, "--cert", cert);
-		args_add_strv (args, "--key", key);
+		if (nmovpn_arg_is_set (cert))
+			args_add_strv (args, "--cert", cert);
+		if (nmovpn_arg_is_set (key))
+			args_add_strv (args, "--key", key);
 	}
 }
 
@@ -548,8 +553,9 @@ openvpn_binary_detect_version (const char *exepath)
 		return OPENVPN_BINARY_VERSION_UNKNOWN;
 
 	if (   !WIFEXITED (exit_code)
-	    || WEXITSTATUS (exit_code) != 1) {
-		/* expect return code 1 (OPENVPN_EXIT_STATUS_USAGE) */
+	    || !NM_IN_SET(WEXITSTATUS (exit_code), 0, 1)) {
+		/* expect return code 1 (OPENVPN_EXIT_STATUS_USAGE).
+		 * Since 2.5.0, it returns 0. */
 		return OPENVPN_BINARY_VERSION_UNKNOWN;
 	}
 
@@ -572,14 +578,32 @@ openvpn_binary_detect_version (const char *exepath)
 
 	if (n <= 3)
 		return OPENVPN_BINARY_VERSION_2_3_OR_OLDER;
+	if (n <= 4)
+		return OPENVPN_BINARY_VERSION_2_4_OR_OLDER;
+
 	return OPENVPN_BINARY_VERSION_2_4_OR_NEWER;
 }
 
 static OpenvpnBinaryVersion
 openvpn_binary_detect_version_cached (const char *exepath, OpenvpnBinaryVersion *cached)
 {
-	if (G_UNLIKELY (*cached == OPENVPN_BINARY_VERSION_INVALID))
+	if (G_UNLIKELY (*cached == OPENVPN_BINARY_VERSION_INVALID)) {
+		const char *str;
+
 		*cached = openvpn_binary_detect_version (exepath);
+		switch (*cached) {
+		case OPENVPN_BINARY_VERSION_2_3_OR_OLDER:
+			str = "2.3 or older";
+			break;
+		case OPENVPN_BINARY_VERSION_2_4_OR_NEWER:
+			str = "2.4 or newer";
+			break;
+		default:
+			str = "unknown";
+			break;
+		}
+		_LOGI ("detected openvpn version %s", str);
+	}
 	return *cached;
 }
 
@@ -1320,6 +1344,7 @@ nm_openvpn_start_openvpn_binary (NMOpenvpnPlugin *plugin,
 	GPid pid;
 	gboolean dev_type_is_tap;
 	const char *defport, *proto_tcp;
+	const char *allow_compression = NULL;
 	const char *compress;
 	const char *tls_remote = NULL;
 	const char *nm_openvpn_user, *nm_openvpn_group, *nm_openvpn_chroot;
@@ -1331,6 +1356,7 @@ nm_openvpn_start_openvpn_binary (NMOpenvpnPlugin *plugin,
 	guint num_remotes = 0;
 	gs_free char *cmd_log = NULL;
 	NMOvpnComp comp;
+	NMOvpnAllowCompression allow_comp;
 
 	s_vpn = nm_connection_get_setting_vpn (connection);
 	if (!s_vpn) {
@@ -1505,6 +1531,9 @@ nm_openvpn_start_openvpn_binary (NMOpenvpnPlugin *plugin,
 	 * See bgo#769177
 	 */
 
+	/* New (2.5+) allow-compression option ("yes", "no", "asym") */
+	allow_compression = nm_setting_vpn_get_data_item (s_vpn,
+	                                                  NM_OPENVPN_KEY_ALLOW_COMPRESSION);
 	/* New (2.4+) compress option ("lz4", "lzo", ...) */
 	compress = nm_setting_vpn_get_data_item (s_vpn, NM_OPENVPN_KEY_COMPRESS);
 	/* Legacy option ("yes", "adaptive", "no", ...) */
@@ -1513,43 +1542,53 @@ nm_openvpn_start_openvpn_binary (NMOpenvpnPlugin *plugin,
 	if (compress && tmp)
 		_LOGW ("'compress' option overrides 'comp-lzo'");
 
+	allow_comp = nmovpn_allow_compression_from_options (allow_compression);
 	comp = nmovpn_compression_from_options (tmp, compress);
 	openvpn_binary_detect_version_cached (openvpn_binary, &openvpn_binary_version);
 
-	switch (comp) {
-	case NMOVPN_COMP_DISABLED:
-		break;
-	case NMOVPN_COMP_LZO:
-		if (openvpn_binary_version == OPENVPN_BINARY_VERSION_2_4_OR_NEWER)
-			args_add_strv (args, "--compress", "lzo");
-		else
-			args_add_strv (args, "--comp-lzo", "yes");
-		break;
-	case NMOVPN_COMP_LZ4:
-	case NMOVPN_COMP_LZ4_V2:
-	case NMOVPN_COMP_AUTO:
-		if (openvpn_binary_version != OPENVPN_BINARY_VERSION_2_4_OR_NEWER)
-			_LOGW ("\"compress\" option supported only by OpenVPN >= 2.4");
-
-		if (comp == NMOVPN_COMP_LZ4)
-			args_add_strv (args, "--compress", "lz4");
-		else if (comp == NMOVPN_COMP_LZ4_V2)
-			args_add_strv (args, "--compress", "lz4-v2");
-		else
-			args_add_strv (args, "--compress");
-		break;
-	case NMOVPN_COMP_LEGACY_LZO_DISABLED:
-	case NMOVPN_COMP_LEGACY_LZO_ADAPTIVE:
-		if (openvpn_binary_version == OPENVPN_BINARY_VERSION_2_4_OR_NEWER)
-			_LOGW ("\"comp-lzo\" is deprecated and will be removed in future OpenVPN releases");
-
-		args_add_strv (args, "--comp-lzo",
-		                  comp == NMOVPN_COMP_LEGACY_LZO_DISABLED
-		               ? "no"
-		               : "adaptive");
-		break;
+	if (nmovpn_arg_is_set (allow_compression)) {
+		if (openvpn_binary_version == OPENVPN_BINARY_VERSION_2_4_OR_OLDER) {
+			_LOGW ("\"allow-compression\" is only supported in OpenVPN 2.5 and later versions");
+		} else {
+			args_add_strv (args, "--allow-compression", allow_compression);
+		}
 	}
 
+	if (allow_comp != NMOVPN_ALLOW_COMPRESSION_NO)
+		switch (comp) {
+		case NMOVPN_COMP_DISABLED:
+			break;
+		case NMOVPN_COMP_LZO:
+			if (openvpn_binary_version == OPENVPN_BINARY_VERSION_2_3_OR_OLDER)
+				args_add_strv (args, "--comp-lzo", "yes");
+			else
+				args_add_strv (args, "--compress", "lzo");
+			break;
+		case NMOVPN_COMP_LZ4:
+		case NMOVPN_COMP_LZ4_V2:
+		case NMOVPN_COMP_AUTO:
+			if (openvpn_binary_version == OPENVPN_BINARY_VERSION_2_3_OR_OLDER)
+				_LOGW ("\"compress\" option supported only by OpenVPN >= 2.4");
+
+			if (comp == NMOVPN_COMP_LZ4)
+				args_add_strv (args, "--compress", "lz4");
+			else if (comp == NMOVPN_COMP_LZ4_V2)
+				args_add_strv (args, "--compress", "lz4-v2");
+			else
+				args_add_strv (args, "--compress");
+			break;
+		case NMOVPN_COMP_LEGACY_LZO_DISABLED:
+		case NMOVPN_COMP_LEGACY_LZO_ADAPTIVE:
+			if (openvpn_binary_version != OPENVPN_BINARY_VERSION_2_3_OR_OLDER)
+				_LOGW ("\"comp-lzo\" is deprecated and will be removed in future OpenVPN releases");
+
+			args_add_strv (args, "--comp-lzo",
+			               comp == NMOVPN_COMP_LEGACY_LZO_DISABLED
+			               ? "no"
+			               : "adaptive");
+			break;
+		}
+
 	tmp = nm_setting_vpn_get_data_item (s_vpn, NM_OPENVPN_KEY_FLOAT);
 	if (nm_streq0 (tmp, "yes"))
 		args_add_strv (args, "--float");
@@ -1708,8 +1747,11 @@ nm_openvpn_start_openvpn_binary (NMOpenvpnPlugin *plugin,
 	}
 	tmp = nm_setting_vpn_get_data_item (s_vpn, NM_OPENVPN_KEY_TLS_VERSION_MIN);
 	if (nmovpn_arg_is_set (tmp)) {
+		const char *or_highest = nm_setting_vpn_get_data_item (s_vpn, NM_OPENVPN_KEY_TLS_VERSION_MIN_OR_HIGHEST);
+
 		args_add_strv (args, "--tls-version-min");
-		args_add_strv (args, tmp);
+		args_add_strv0 (args, tmp, nm_streq0(or_highest, "yes") ? "or-highest" : NULL);
+
 	}
 	tmp = nm_setting_vpn_get_data_item (s_vpn, NM_OPENVPN_KEY_TLS_VERSION_MAX);
 	if (nmovpn_arg_is_set (tmp)) {
@@ -1725,11 +1767,11 @@ nm_openvpn_start_openvpn_binary (NMOpenvpnPlugin *plugin,
 
 	tmp = nm_setting_vpn_get_data_item (s_vpn, NM_OPENVPN_KEY_TLS_REMOTE);
 	if (nmovpn_arg_is_set (tmp)) {
-		if (openvpn_binary_detect_version_cached (openvpn_binary, &openvpn_binary_version) != OPENVPN_BINARY_VERSION_2_4_OR_NEWER) {
-			_LOGW ("the tls-remote option is deprecated and removed from OpenVPN 2.4. Update your connection to use verify-x509-name");
+		if (openvpn_binary_detect_version_cached (openvpn_binary, &openvpn_binary_version) == OPENVPN_BINARY_VERSION_2_3_OR_OLDER) {
+			_LOGW ("the tls-remote option is deprecated and removed from OpenVPN 2.4. Update your connection to use verify-x509-name (for example, \"verify-x509-name=name:%s\")", tmp);
 			args_add_strv (args, "--tls-remote", tmp);
 		} else {
-			_LOGW ("the tls-remote option is deprecated and removed from OpenVPN 2.4. For compatibility, the plugin uses \"verify-x509-name\" \"%s\" \"name\" instead. Update your connection to use verify-x509-name", tmp);
+			_LOGW ("the tls-remote option is deprecated and removed from OpenVPN 2.4. For compatibility, the plugin uses \"verify-x509-name\" \"%s\" \"name\" instead. Update your connection to use for example \"verify-x509-name=name:%s\")", tmp, tmp);
 			args_add_strv (args, "--verify-x509-name", tmp, "name");
 		}
 		tls_remote = tmp;
